/// <reference types="@lorisleiva/js-next-alpha/node_modules/@solana/web3.js" />
import type { ErrorResolver } from '@metaplex-foundation/cusper';
import { Connection, Keypair, PublicKey } from '@solana/web3.js';
import { AddressLabels, GenKeypair } from './diagnostics/address-labels';
import { AmmanClient, AmmanClientOpts } from './relay';
import { AmmanMockStorageDriverOptions } from './storage';
import { PayerTransactionHandler, TransactionLabelMapper } from './transactions/transaction-handler';
/**
 * Creates an Amman instance which is used to interact with address labels and
 * other amman features.
 * By default it connects the socket client to the running amman validator.
 * This allows it to update the amman-explorer with recent transactions and addresses.
 *
 * NOTE: that Amman should only be instantiated once during the life time of
 * your program, tests, etc.
 *
 * ## Example
 *
 * ```js
 * export const amman = Amman.instance({
 *   knownLabels: { [PROGRAM_ADDRESS]: 'My Program' },
 *   logLabel: console.log,
 * })
 * ```
 *
 */
export declare class Amman {
    /**
     * Exposes the {@link AddressLabels} API to add and query labels for
     * addresses of accounts and transactions.
     */
    readonly addr: AddressLabels;
    readonly ammanClient: AmmanClient;
    readonly errorResolver?: ErrorResolver | undefined;
    private constructor();
    private static _instance;
    /**
     * Generates a keypair and returns its public key and the keypair itself as a Tuple.
     *
     * @param label if provided the key will be added to existing labels
     * @return [publicKey, keypair ]
     */
    genKeypair: GenKeypair;
    /**
     * Drops the specified amount of tokens to the provided public key.
     *
     * @param connection to solana JSON RPC node
     * @param publicKey to drop sols to
     * @param sol amount of sols to drop
     *
     * @category utils
     */
    airdrop(connection: Connection, publicKey: PublicKey, sol?: number): Promise<{
        signature: string;
        signatureResult: import("@solana/web3.js").RpcResponseAndContext<import("@solana/web3.js").SignatureResult>;
    }>;
    /**
     * Provides a {@link TransactionHandler} which uses the {@link payer} to sign transactions.
     */
    payerTransactionHandler(connection: Connection, payer: Keypair): PayerTransactionHandler;
    /**
     * Provides a {@link AmmanMockStorageDriver} which stores uploaded data on
     * the filesystem inside a tmp directory.
     * The {@link MockStorageServer} initialized with the same {@link storageId}
     * serves the files from there.
     *
     * @category storage
     */
    createMockStorageDriver: (storageId: string, uploadRoot: string, options?: AmmanMockStorageDriverOptions | undefined) => import("@lorisleiva/js-next-alpha").MetaplexPlugin;
    /**
     * Disconnects the amman relay client and allows the app to shut down.
     * Only needed if you set `{ autoUnref: false }` for the amman client opts.
     */
    disconnect(): void;
    /**
     * Creates an instance of {@link Amman}.
     *
     * @param args
     * @param args.knownLabels label keys that do not change, i.e. `{ [PROGRM_ID]:  'My Program' }`
     * @param args.log used to log labels that are added to {@link
     * Amman.addresses} and information about other events
     * @param args.connectClient used to determine if to connect an amman client
     * if no {@link args.ammanClient} is provided; defaults to connect unless running in a CI environment
     * @param args.ammanClient allows to override the client used to connect to the amman validator
     * @param args.ammanClientOpts allows to specify options for the amman relay client instead
     * @param args.errorResolver used to resolve a known errors
     * from the program logs, see {@link https://github.com/metaplex-foundation/cusper}
     * @param args.transactionLabelMapper function to replace key strings in a
     * label to enhance it, i.e. with an icon
     */
    static instance(args?: {
        knownLabels?: Record<string, string>;
        log?: (msg: string) => void;
        ammanClient?: AmmanClient;
        connectClient?: boolean;
        ammanClientOpts?: AmmanClientOpts;
        errorResolver?: ErrorResolver;
        transactionLabelMapper?: TransactionLabelMapper;
    }): Amman;
    /** @internal */
    static get existingInstance(): Amman | undefined;
}
