import { TransactionResponse } from '@solana/web3.js';
import { ConfirmedTransactionDetails, MaybeErrorWithCode, TransactionSummary } from '../transactions';
/**
 * The minimum methods that the first argument passed to assert functions like
 * {@link assertConfirmedTransaction} needs to have.
 *
 * @category asserts
 */
export declare type Assert = {
    equal(actual: any, expected: any, msg?: string): void;
    ok(value: any, msg?: string): void;
    fail(msg?: string): void;
    match(actual: string, expected: RegExp, msg?: string): void;
};
/**
 * Asserts details about a confirmed transaction
 *
 * @param t
 * @param tx the confirmed transaction to verify
 * @param args specify what details should be verified
 * @category asserts
 */
export declare function assertConfirmedTransaction(t: Assert, tx: TransactionResponse, args?: {
    fee?: number;
}): void;
/**
 * Asserts details about a {@link TransactionSummary}.
 *
 * @param t
 * @param summary transaction summary to verify
 * @param args specify what details should be verified
 * @category asserts
 */
export declare function assertTransactionSummary(t: Assert, summary: TransactionSummary, args?: {
    fee?: number;
    msgRx?: RegExp[];
}): void;
/**
 * Asserts details about the provided error.
 *
 * @param t
 * @param err error to verify
 * @param msgRxs list of {@link RegExp} which will be matched on the error _message_ or `err.logs`.
 * @category asserts
 */
export declare function assertError(t: Assert, err: Error & {
    logs?: string[];
}, msgRxs: RegExp[]): void;
/**
 * Asserts that the provided error matches the expected one by verifying the
 * error type and optionally the error message.
 *
 * @param t
 * @param err error to verify
 * @param ty the type of the error to expect
 * @param msgRx a {@link RegExp} that the error message is expected to match
 */
export declare function assertMatchesError<Err extends Function>(t: Assert, err: MaybeErrorWithCode, ty: Err, msgRx?: RegExp): void;
/**
 * Asserts that the provided {@link ConfirmedTransactionDetails} has an error
 * that matches the expected one by verifying the error type and optionally the
 * error message.
 *
 * @param t
 * @param res result of executing a transaction
 * @param ty the type of the error to expect
 * @param msgRx a {@link RegExp} that the error message is expected to match
 */
export declare function assertHasError<Err extends Function>(t: Assert, res: ConfirmedTransactionDetails, ty: Err, msgRx?: RegExp): void;
