"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertHasError = exports.assertMatchesError = exports.assertError = exports.assertTransactionSummary = exports.assertConfirmedTransaction = void 0;
/**
 * Asserts details about a confirmed transaction
 *
 * @param t
 * @param tx the confirmed transaction to verify
 * @param args specify what details should be verified
 * @category asserts
 */
function assertConfirmedTransaction(t, tx, args = {}) {
    var _a, _b;
    t.equal((_a = tx.meta) === null || _a === void 0 ? void 0 : _a.err, null, 'confirmed transaction has no error');
    if (args.fee != null) {
        t.equal((_b = tx.meta) === null || _b === void 0 ? void 0 : _b.fee, args.fee, 'confirmed transaction fee matches');
    }
}
exports.assertConfirmedTransaction = assertConfirmedTransaction;
/**
 * Asserts details about a {@link TransactionSummary}.
 *
 * @param t
 * @param summary transaction summary to verify
 * @param args specify what details should be verified
 * @category asserts
 */
function assertTransactionSummary(t, summary, args = {}) {
    t.equal(summary.transactionError, null, 'transaction summary has no transaction error');
    if (args.fee != null) {
        t.equal(summary.fee, args.fee, 'transaction summary fee matches');
    }
    if (args.msgRx != null) {
        for (const msgRx of args.msgRx) {
            const hasMatch = summary.logMessages.some((x) => msgRx.test(x));
            if (!hasMatch) {
                console.error('Failed to find %s inside', msgRx.toString());
                console.error(summary.logMessages.join('\n  '));
            }
            t.ok(hasMatch, `match '${msgRx.toString()}' in transaction summary log messages`);
        }
    }
}
exports.assertTransactionSummary = assertTransactionSummary;
/**
 * Asserts details about the provided error.
 *
 * @param t
 * @param err error to verify
 * @param msgRxs list of {@link RegExp} which will be matched on the error _message_ or `err.logs`.
 * @category asserts
 */
function assertError(t, err, msgRxs) {
    var _a;
    t.ok(err != null, 'error encountered');
    const errorMessages = err
        .toString()
        .split('\n')
        .concat((_a = err.logs) !== null && _a !== void 0 ? _a : []);
    for (const msgRx of msgRxs) {
        const hasMatch = errorMessages.some((x) => msgRx.test(x));
        if (!hasMatch) {
            console.error('Failed to find %s inside', msgRx.toString());
            console.error(errorMessages.join('\n  '));
        }
        t.ok(hasMatch, `match '${msgRx.toString()}' in error message`);
    }
}
exports.assertError = assertError;
/**
 * Asserts that the provided error matches the expected one by verifying the
 * error type and optionally the error message.
 *
 * @param t
 * @param err error to verify
 * @param ty the type of the error to expect
 * @param msgRx a {@link RegExp} that the error message is expected to match
 */
function assertMatchesError(t, err, ty, msgRx) {
    if (err == null) {
        t.fail(`Expected an error of type ${ty}`);
        return;
    }
    t.ok(err instanceof ty, ty.name);
    if (msgRx != null) {
        t.match(err.message, msgRx);
    }
}
exports.assertMatchesError = assertMatchesError;
/**
 * Asserts that the provided {@link ConfirmedTransactionDetails} has an error
 * that matches the expected one by verifying the error type and optionally the
 * error message.
 *
 * @param t
 * @param res result of executing a transaction
 * @param ty the type of the error to expect
 * @param msgRx a {@link RegExp} that the error message is expected to match
 */
function assertHasError(t, res, ty, msgRx) {
    return assertMatchesError(t, res.txSummary.err, ty, msgRx);
}
exports.assertHasError = assertHasError;
//# sourceMappingURL=index.js.map