"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleAccountCommand = void 0;
const web3_js_1 = require("@solana/web3.js");
const utils_1 = require("../../utils");
const assert_1 = require("assert");
const ansi_colors_1 = require("ansi-colors");
// @ts-ignore no types available, but it's a simpler function
const buffer_hexdump_1 = __importDefault(require("buffer-hexdump"));
const text_table_1 = __importDefault(require("text-table"));
const providers_1 = require("../../accounts/providers");
const utils_2 = require("../utils");
async function handleAccountCommand(acc, includeTxs = false) {
    var _a;
    if (acc == null)
        return renderAllKnownAccounts(includeTxs);
    const amman = (0, utils_2.cliAmmanInstance)();
    const addresses = await (0, utils_2.resolveAccountAddresses)(amman, acc);
    if (addresses.length === 0) {
        throw new Error(`Account ${acc} could not be resolved to an address`);
    }
    const connection = new web3_js_1.Connection(utils_1.LOCALHOST, 'singleGossip');
    const rendereds = [];
    for (const address of addresses) {
        const pubkey = new web3_js_1.PublicKey(address);
        const accountInfo = await connection.getAccountInfo(pubkey);
        (0, assert_1.strict)(accountInfo != null, 'Account info should not be null');
        const len = accountInfo.data.length;
        const sol = accountInfo.lamports / web3_js_1.LAMPORTS_PER_SOL;
        const accountData = (_a = (await tryResolveAccountData(pubkey))) !== null && _a !== void 0 ? _a : '';
        const rendered = `
${(0, ansi_colors_1.bold)('Public Key')}: ${address}
${(0, ansi_colors_1.bold)('Balance   ')}: ${sol} SOL
${(0, ansi_colors_1.bold)('Owner     ')}: ${accountInfo.owner}
${(0, ansi_colors_1.bold)('Executable')}: ${accountInfo.executable}
${(0, ansi_colors_1.bold)('Rent Epoch')}: ${accountInfo.rentEpoch}
Length: ${len} (0x${len.toString(16)}) bytes
${(0, buffer_hexdump_1.default)(accountInfo.data)}
${accountData}
`;
        rendereds.push(rendered);
    }
    let rendered = rendereds.join('\n==================================================================\n');
    if (rendereds.length > 1) {
        rendered +=
            `\n${(0, ansi_colors_1.bold)('NOTE')}: found ${rendereds.length}` +
                ` accounts labeled '${acc}' and printed all of them above`;
    }
    amman.disconnect();
    return { connection, rendered };
}
exports.handleAccountCommand = handleAccountCommand;
async function renderAllKnownAccounts(includeTxs) {
    const amman = (0, utils_2.cliAmmanInstance)();
    const accounts = await amman.addr.getRemoteLabelAddresses();
    if (Object.keys(accounts).length === 0) {
        const rendered = 'No labeled accounts found';
        return { connection: undefined, rendered };
    }
    const rows = [];
    for (const [address, label] of Object.entries(accounts)) {
        if (!includeTxs && address.length > 44)
            continue;
        rows.push([(0, ansi_colors_1.bold)(label), address]);
    }
    const rendered = (0, text_table_1.default)(rows);
    amman.disconnect();
    return { connection: undefined, rendered };
}
async function tryResolveAccountData(pubkey) {
    var _a;
    const accountProvider = providers_1.AccountProvider.fromRecord({}, new Map());
    const res = await accountProvider.syncAccountInformation(pubkey);
    const pretty = (_a = res === null || res === void 0 ? void 0 : res.account) === null || _a === void 0 ? void 0 : _a.pretty();
    if (pretty == null)
        return;
    const rows = [];
    for (const [key, value] of Object.entries(pretty)) {
        rows.push([key, (0, ansi_colors_1.dim)(value !== null && value !== void 0 ? value : 'null')]);
    }
    return (`\n${(0, ansi_colors_1.bold)('Account Data')}` +
        `\n${(0, ansi_colors_1.bold)('------------')}` +
        `\n${(0, text_table_1.default)(rows)}`);
}
//# sourceMappingURL=account.js.map