"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DisconnectedAmmanClient = exports.ConnectedAmmanClient = void 0;
const socket_io_client_1 = __importDefault(require("socket.io-client"));
const utils_1 = require("../utils");
const consts_1 = require("./consts");
const timeout_1 = require("./timeout");
const AMMAN_UNABLE_ADD_LABELS = 'Unable to connect to send address labels';
const AMMAN_UNABLE_FETCH_LABELS = 'Unable to connect to fetch address labels';
const AMMAN_NOT_RUNNING_ERROR = ', is amman running?\n';
'If not please start one in a separate terminal via `amman start`.\n' +
    'Alternatively do not set the `ack` option to `true` when instantiating the amman instance.';
/** @private */
class ConnectedAmmanClient {
    constructor(url, opts = {}) {
        this.url = url;
        const { autoUnref = true, ack = false } = opts;
        this.ack = ack;
        this.socket = (0, socket_io_client_1.default)(url, { autoUnref });
    }
    connect() {
        if (this.socket.connected)
            return this;
        this.socket.connect();
        (0, utils_1.logDebug)('AmmanClient connected');
        return this;
    }
    clearAddressLabels() {
        this.socket.emit(consts_1.MSG_CLEAR_ADDRESS_LABELS);
    }
    clearTransactions() {
        this.socket.emit(consts_1.MSG_CLEAR_TRANSACTIONS);
    }
    addAddressLabels(labels) {
        if (utils_1.logTrace.enabled) {
            const labelCount = Object.keys(labels).length;
            (0, utils_1.logTrace)(`Adding ${labelCount} address labels`);
        }
        const promise = this.ack
            ? new Promise((resolve, reject) => {
                const timeout = (0, timeout_1.createTimeout)(2000, AMMAN_UNABLE_ADD_LABELS + AMMAN_NOT_RUNNING_ERROR, reject);
                this.socket
                    .on('error', (err) => {
                    clearTimeout(timeout);
                    reject(err);
                })
                    .on(consts_1.ACK_UPDATE_ADDRESS_LABELS, () => {
                    (0, utils_1.logTrace)('Got ack for address labels update %O', labels);
                    clearTimeout(timeout);
                    resolve();
                });
            })
            : Promise.resolve();
        this.socket.emit(consts_1.MSG_UPDATE_ADDRESS_LABELS, labels);
        return promise;
    }
    async fetchAddressLabels() {
        (0, utils_1.logTrace)('Fetching address labels');
        return new Promise((resolve, reject) => {
            const timeout = (0, timeout_1.createTimeout)(2000, AMMAN_UNABLE_FETCH_LABELS + AMMAN_NOT_RUNNING_ERROR, reject);
            this.socket
                .on('error', (err) => {
                clearTimeout(timeout);
                reject(err);
            })
                .on(consts_1.MSG_UPDATE_ADDRESS_LABELS, (labels) => {
                clearTimeout(timeout);
                (0, utils_1.logTrace)('Got address labels %O', labels);
                resolve(labels);
            })
                .emit(consts_1.MSG_GET_KNOWN_ADDRESS_LABELS);
        });
    }
    /**
     * Disconnects this client and allows the app to shut down.
     * Only needed if you set `{ autoUnref: false }` for the opts.
     */
    disconnect() {
        this.socket.disconnect();
    }
    static getInstance(url, ammanClientOpts) {
        if (ConnectedAmmanClient._instance != null)
            return ConnectedAmmanClient._instance;
        ConnectedAmmanClient._instance = new ConnectedAmmanClient(url !== null && url !== void 0 ? url : consts_1.AMMAN_RELAY_URI, ammanClientOpts).connect();
        return ConnectedAmmanClient._instance;
    }
}
exports.ConnectedAmmanClient = ConnectedAmmanClient;
/** @private */
class DisconnectedAmmanClient {
    clearAddressLabels() { }
    clearTransactions() { }
    addAddressLabels(_labels) {
        return Promise.resolve();
    }
    fetchAddressLabels() {
        return Promise.resolve({});
    }
    disconnect() { }
}
exports.DisconnectedAmmanClient = DisconnectedAmmanClient;
//# sourceMappingURL=client.js.map