"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Relay = void 0;
const http_1 = require("http");
const socket_io_1 = require("socket.io");
const providers_1 = require("../accounts/providers");
const utils_1 = require("../utils");
const http_2 = require("../utils/http");
const consts_1 = require("./consts");
/**
 * A simple socket.io server which communicates to the Amman Explorere as well as accepting connections
 * from other clients, i.e. via an {@link AmmanClient} which tests can use to communicate via the amman API.
 *
 * @private
 */
class RelayServer {
    constructor(io, accountProvider, 
    // Keyed pubkey:label
    allKnownLabels = {}) {
        this.io = io;
        this.accountProvider = accountProvider;
        this.allKnownLabels = allKnownLabels;
        this.hookConnectionEvents();
    }
    hookConnectionEvents() {
        this.io.on('connection', (socket) => {
            const client = `${socket.id} from ${socket.client.conn.remoteAddress}`;
            socket.on('disconnect', () => (0, utils_1.logDebug)(`socket.io ${client} disconnected`));
            (0, utils_1.logDebug)(`socket.io ${client} connected`);
            this.hookMessages(socket);
        });
    }
    hookMessages(socket) {
        socket
            .on(consts_1.MSG_UPDATE_ADDRESS_LABELS, (labels) => {
            if (utils_1.logTrace.enabled) {
                const labelCount = Object.keys(labels).length;
                (0, utils_1.logTrace)(`Got ${labelCount} labels, broadcasting ...`);
            }
            for (const [key, val] of Object.entries(labels)) {
                this.allKnownLabels[key] = val;
            }
            socket.broadcast.emit(consts_1.MSG_UPDATE_ADDRESS_LABELS, labels);
            socket.emit(consts_1.ACK_UPDATE_ADDRESS_LABELS);
        })
            .on(consts_1.MSG_GET_KNOWN_ADDRESS_LABELS, () => {
            if (utils_1.logTrace.enabled) {
                const labelCount = Object.keys(this.allKnownLabels).length;
                (0, utils_1.logTrace)(`Sending ${labelCount} known labels to requesting client.`);
            }
            socket.emit(consts_1.MSG_UPDATE_ADDRESS_LABELS, this.allKnownLabels);
        })
            .on(consts_1.MSG_WATCH_ACCOUNT_INFO, async (accountAddress) => {
            this.accountProvider.watchAccount(accountAddress, (account, rendered) => {
                if (socket.disconnected)
                    return;
                const pretty = account.pretty();
                if (utils_1.logTrace.enabled) {
                    (0, utils_1.logTrace)(`Sending account ${(0, utils_1.safeJsonStringify)({
                        pretty,
                        rendered,
                    })} to ${socket.conn.remoteAddress}`);
                }
                socket.broadcast.emit(consts_1.MSG_UPDATE_ACCOUNT_INFO, {
                    accountAddress,
                    accountInfo: { pretty, rendered },
                });
            });
        });
    }
}
/**
 * Sets up the Amman Relay which uses the given account provider to resolve account data.
 * @private
 * */
class Relay {
    static createApp(accountProvider, knownLabels) {
        const server = (0, http_1.createServer)();
        const io = new socket_io_1.Server(server, {
            cors: {
                origin: '*',
            },
        });
        const relayServer = new RelayServer(io, accountProvider, knownLabels);
        return { app: server, io, relayServer };
    }
    static async startServer(accountProviders, accountRenderers, programs, killRunning = true) {
        if (killRunning) {
            await (0, http_2.killRunningServer)(consts_1.AMMAN_RELAY_PORT);
        }
        const accountProvider = providers_1.AccountProvider.fromRecord(accountProviders, accountRenderers);
        const knownLabels = programs
            .filter((x) => x.label != null)
            .reduce((acc, x) => {
            acc[x.programId] = x.label;
            return acc;
        }, {});
        const { app, io, relayServer } = this.createApp(accountProvider, knownLabels);
        return new Promise((resolve, reject) => {
            app.on('error', reject).listen(consts_1.AMMAN_RELAY_PORT, () => {
                const addr = app.address();
                const msg = `Amman Relay listening on ${addr.address}:${addr.port}`;
                (0, utils_1.logDebug)(msg);
                resolve({ app, io, relayServer });
            });
        });
    }
}
exports.Relay = Relay;
//# sourceMappingURL=server.js.map