"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AmmanMockStorageDriver = void 0;
const js_next_alpha_1 = require("@lorisleiva/js-next-alpha");
const assert_1 = require("assert");
const bn_js_1 = __importDefault(require("bn.js"));
const path_1 = __importDefault(require("path"));
const utils_1 = require("../utils");
const fs_1 = require("../utils/fs");
const consts_1 = require("./consts");
const fs_2 = require("fs");
const DEFAULT_COST_PER_BYTE = new bn_js_1.default(1);
class AmmanMockStorageDriver extends js_next_alpha_1.StorageDriver {
    constructor(metaplex, storageId, uploadRoot, costPerByte, logInfo, logDebug, logTrace) {
        super(metaplex);
        this.storageId = storageId;
        this.uploadRoot = uploadRoot;
        this.costPerByte = costPerByte;
        this.logInfo = logInfo;
        this.logDebug = logDebug;
        this.logTrace = logTrace;
        this.cache = {};
        (0, fs_1.assertValidPathSegmentWithoutSpaces)(storageId, 'please select a different storage id');
        this.storageDir = path_1.default.join(consts_1.AMMAN_STORAGE_ROOT, storageId);
        (0, fs_1.ensureDirSync)(this.storageDir);
        (0, assert_1.strict)((0, fs_1.canAccessSync)(this.uploadRoot), `uploadRoot '${uploadRoot}' must be accessible, but is not`);
        this.baseUrl = AmmanMockStorageDriver.getStorageUri(storageId);
        this.logInfo(`Amman Storage Driver with '${storageId}' initialized`);
        this.logDebug({
            uploadRoot,
            storageDir: this.storageDir,
            baseUrl: this.baseUrl,
        });
    }
    async getPrice(file) {
        return new bn_js_1.default(file.buffer.byteLength).mul(this.costPerByte);
    }
    async upload(file) {
        this.logTrace(file);
        const resourceUri = file.uniqueName;
        const uri = `${this.baseUrl}/${resourceUri}`;
        const fullDst = path_1.default.join(this.storageDir, resourceUri);
        // JSON files include inline metadata instead of referencing an image to upload
        if (file.contentType === 'application/json') {
            await fs_2.promises.writeFile(fullDst, file.toBuffer());
        }
        else {
            // Copy from upload dir into storage
            const fullSrc = path_1.default.join(this.uploadRoot, file.fileName);
            await fs_2.promises.copyFile(fullSrc, fullDst);
        }
        this.logDebug(`Uploaded ${file.displayName}:${file.uniqueName} to ${fullDst}`);
        this.cache[uri] = file;
        return uri;
    }
    async download(uri) {
        const file = this.cache[uri];
        (0, assert_1.strict)(file != null, `file '${uri}' not found`);
        return file;
    }
    async downloadJson(uri) {
        const file = await this.download(uri);
        return JSON.parse(file.toString());
    }
}
exports.AmmanMockStorageDriver = AmmanMockStorageDriver;
AmmanMockStorageDriver.create = (storageId, uploadRoot, options = {}) => {
    const { costPerByte = DEFAULT_COST_PER_BYTE, logInfo = utils_1.logInfo, logDebug = utils_1.logDebug, logTrace = utils_1.logTrace, } = options;
    return {
        install: (metaplex) => metaplex.setStorage(new AmmanMockStorageDriver(metaplex, storageId, uploadRoot, new bn_js_1.default(costPerByte), logInfo, logDebug, logTrace)),
    };
};
AmmanMockStorageDriver.getStorageUri = (storageId) => `${consts_1.AMMAN_STORAGE_URI}/${storageId}`;
//# sourceMappingURL=mock-driver.js.map