"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PayerTransactionHandler = void 0;
const assert_1 = require("assert");
const _1 = require(".");
const address_labels_1 = require("../diagnostics/address-labels");
function transactionSummary(tx, errorResolver) {
    var _a, _b, _c, _d, _e, _f;
    const logMessages = (_b = (_a = tx.meta) === null || _a === void 0 ? void 0 : _a.logMessages) !== null && _b !== void 0 ? _b : [];
    const fee = (_c = tx.meta) === null || _c === void 0 ? void 0 : _c.fee;
    const slot = tx.slot;
    const blockTime = (_d = tx.blockTime) !== null && _d !== void 0 ? _d : 0;
    const transactionError = (_e = tx.meta) === null || _e === void 0 ? void 0 : _e.err;
    const err = (_f = errorResolver === null || errorResolver === void 0 ? void 0 : errorResolver.errorFromProgramLogs(logMessages, true)) !== null && _f !== void 0 ? _f : undefined;
    return { logMessages, fee, slot, blockTime, transactionError, err };
}
const FAIL = '❌';
function defaultTransactionLabelMapper(label) {
    return label.replace(/^(Fail|Fails|Failure|Err|Error|Bug):?/i, FAIL);
}
/**
 * A {@link TransactionHandler} backed by a payer {@link Keypair}.
 * @category transactions
 */
class PayerTransactionHandler {
    /**
     * Creates a {@link PayerTransactionHandler}.
     *
     * @param connection to use to handle transactions
     * @param payer to use to sign transactions
     * @param errorResolver used to resolve a known error from the program logs
     */
    constructor(connection, payer, errorResolver, transactionLabelMapper = defaultTransactionLabelMapper) {
        this.connection = connection;
        this.payer = payer;
        this.errorResolver = errorResolver;
        this.transactionLabelMapper = transactionLabelMapper;
    }
    /**
     * Public key of the payer
     */
    get publicKey() {
        return this.payer.publicKey;
    }
    /**
     * Sends and confirms the transaction {@link TransactionHandler['sendAndConfirmTransaction']}.
     */
    async sendAndConfirmTransaction(transaction, signers, optionsOrLabel, label) {
        transaction.recentBlockhash = (await this.connection.getLatestBlockhash()).blockhash;
        const optionsIsLabel = typeof optionsOrLabel === 'string';
        const options = optionsIsLabel ? undefined : optionsOrLabel;
        const addressLabel = optionsIsLabel ? optionsOrLabel : label;
        const txSignature = await this.connection.sendTransaction(transaction, [this.payer, ...signers], options !== null && options !== void 0 ? options : _1.defaultSendOptions);
        if (addressLabel != null) {
            address_labels_1.AddressLabels.instance.addLabel(this.transactionLabelMapper(addressLabel), txSignature);
        }
        const txRpcResponse = await this.connection.confirmTransaction(txSignature);
        const txConfirmed = await this.connection.getTransaction(txSignature);
        (0, assert_1.strict)(txConfirmed != null, 'confirmed transaction should not be null');
        const txSummary = transactionSummary(txConfirmed, this.errorResolver);
        return { txSignature, txRpcResponse, txConfirmed, txSummary };
    }
}
exports.PayerTransactionHandler = PayerTransactionHandler;
//# sourceMappingURL=transaction-handler.js.map